# Copyright (c) 2024 - 2025 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import sys
from amd.rocal.plugin.pytorch import ROCALClassificationIterator
from amd.rocal.pipeline import Pipeline
import amd.rocal.types as types
import amd.rocal.fn as fn
import os
import cv2
from parse_config import parse_args


def draw_patches(img, idx, args=None):
    # image is expected as a tensor
    if args.rocal_gpu:
        image = img.cpu().numpy()
    else:
        image = img.detach().numpy()
    if not args.NHWC:
        image = image.transpose([1, 2, 0])
    image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)
    cv2.imwrite("output_folder/web_dataset_reader/" +
                str(idx)+"_"+"train"+".png", image)


def main():
    args = parse_args()
    # Args
    image_path = args.image_dataset_path
    index_file = args.index_path
    rocal_cpu = False if args.rocal_gpu else True
    batch_size = args.batch_size
    num_threads = args.num_threads
    random_seed = args.seed
    device = "gpu" if args.rocal_gpu else "cpu"
    try:
        path = "output_folder/web_dataset_reader/"
        if not os.path.exists(path):
            os.makedirs(path)
    except OSError as error:
        print(error)
    # Create Pipeline instance
    pipe = Pipeline(batch_size=batch_size, num_threads=num_threads,
                    device_id=args.local_rank, seed=random_seed, rocal_cpu=rocal_cpu)
    # Use pipeline instance to make calls to reader, decoder & augmentation's
    with pipe:
        img_raw = fn.readers.webdataset(path=image_path, ext=[{'JPEG', 'cls'}], index_paths=index_file, missing_components_behavior=types.MISSING_COMPONENT_ERROR)
        img = fn.decoders.image(img_raw, file_root=image_path,
                                max_decoded_width=500, max_decoded_height=500, index_path=index_file)
        pipe.set_outputs(img)
    # Build the pipeline
    pipe.build()
    # Dataloader
    data_loader = ROCALClassificationIterator(pipe, display=0, device=device, device_id=args.local_rank)
    # Training loop
    cnt = 0
    # Enumerate over the Dataloader
    for epoch in range(args.num_epochs):  # loop over the dataset multiple times
        print("epoch:: ", epoch)
        for i, ([image_batch], meta_data) in enumerate(data_loader, 0):
            if i == 0 and args.print_tensor:
                print("\r Mini-batch " + str(i))
                print("Images", image_batch)
                print("Meta Data - ASCII", meta_data)
            for element in list(range(batch_size)):
                cnt += 1
                draw_patches(image_batch[element],
                             cnt, args=args)
        data_loader.reset()
    print("##############################  WEBDATASET READER SUCCESS  ############################")


if __name__ == "__main__":
    main()



# Copyright (c) 2018 - 2025 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import sys
from amd.rocal.plugin.pytorch import ROCALClassificationIterator
from amd.rocal.pipeline import Pipeline
import amd.rocal.types as types
import amd.rocal.fn as fn
import os
import cv2
from parse_config import parse_args


def draw_patches(img, idx, bboxes=None, args=None):
    # image is expected as a tensor, bboxes as tensors
    if args.rocal_gpu:
        image = img.cpu().numpy()
    else:
        image = img.detach().numpy()
    if not args.NHWC:
        image = image.transpose([1, 2, 0])
    image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)
    if args.classification:
        cv2.imwrite("output_folder/caffe_reader/classification/" +
                    str(idx)+"_"+"train"+".png", image)
    else:
        if bboxes is not None:
            bboxes = bboxes.detach().numpy()
            for (l, t, r, b) in bboxes:
                loc_ = [l, t, r, b]
                color = (255, 0, 0)
                thickness = 2
                image = cv2.rectangle(image, (int(loc_[0]), int(loc_[1])), (int(
                    (loc_[2])), int((loc_[3]))), color, thickness)
        cv2.imwrite("output_folder/caffe_reader/detection/" +
                    str(idx)+"_"+"train"+".png", image)

def main():
    args = parse_args()
    # Args
    image_path = args.image_dataset_path
    rocal_cpu = False if args.rocal_gpu else True
    batch_size = args.batch_size
    rocal_bbox = False if args.classification else True
    num_threads = args.num_threads
    local_rank = args.local_rank
    world_size = args.world_size
    random_seed = args.seed
    device = "gpu" if args.rocal_gpu else "cpu"
    tensor_layout = types.NHWC if args.NHWC else types.NCHW
    num_classes = len(next(os.walk(image_path))[1])
    try:
        if args.classification:
            path = "output_folder/caffe_reader/classification/"
        else:
            path = "output_folder/caffe_reader/detection/"
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
    except OSError as error:
        print(error)
    print("num_classes:: ", num_classes)
    # Create Pipeline instance
    pipe = Pipeline(batch_size=batch_size, num_threads=num_threads,
                    device_id=args.local_rank, seed=random_seed, rocal_cpu=rocal_cpu)
    # Use pipeline instance to make calls to reader, decoder & augmentation's
    with pipe:
        if rocal_bbox:
            jpegs, labels, bboxes = fn.readers.caffe(
                path=image_path, bbox=rocal_bbox)
            images = fn.decoders.image(
                jpegs, path=image_path, shard_id=local_rank, random_shuffle=True)

        else:
            jpegs, labels = fn.readers.caffe(path=image_path, bbox=rocal_bbox)
            images = fn.decoders.image(jpegs, path=image_path, output_type=types.RGB,
                                       shard_id=local_rank, num_shards=world_size, random_shuffle=True)

        images = fn.resize(images, resize_width=224,
                           resize_height=224, output_layout=tensor_layout)
        pipe.set_outputs(images)
    # Build the pipeline
    pipe.build()
    # Dataloader
    data_loader = ROCALClassificationIterator(
        pipe, display=0, device=device, device_id=args.local_rank)
    # Training loop
    cnt = 0
    # Enumerate over the Dataloader
    for epoch in range(args.num_epochs):  # loop over the dataset multiple times
        print("epoch:: ", epoch)
        if not rocal_bbox:
            for i, ([image_batch], labels) in enumerate(data_loader, 0):  # Classification
                if args.print_tensor:
                    sys.stdout.write("\r Mini-batch " + str(i))
                    print("Images", image_batch)
                    print("Labels", labels)
                for element in list(range(batch_size)):
                    cnt += 1
                    draw_patches(image_batch[element], cnt, args=args)
            data_loader.reset()
        else:
            for i, ([image_batch], bboxes, labels) in enumerate(data_loader, 0):  # Detection
                if i == 0:
                    if args.print_tensor:
                        sys.stdout.write("\r Mini-batch " + str(i))
                        print("Images", image_batch)
                        print("Bboxes", bboxes)
                        print("Labels", labels)
                for element in list(range(batch_size)):
                    cnt += 1
                    draw_patches(image_batch[element],
                                 cnt, bboxes[element], args=args)
            data_loader.reset()
    print("##############################  CAFFE READER (CLASSIFCATION/ DETECTION)  SUCCESS  ############################")

if __name__ == "__main__":
    main()



# Copyright (c) 2024 - 2025 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import os
import sys
import subprocess  # nosec
import shutil
import argparse

test_case_augmentation_map = {
    0: "audio_decoder",
    1: "preemphasis_filter",
    2: "spectrogram",
    3: "downmix",
    4: "to_decibels",
    5: "resample",
    6: "tensor_add_tensor",
    7: "tensor_mul_scalar",
    8: "non_silent_region",
    9: "slice",
    10: "mel_filter_bank",
    11: "normalize"
}

def run_unit_test(rocal_data_path, qa_mode, gpu, downmix, build_folder_path, case_list):
    passed_cases = []
    failed_cases = []
    num_passed = 0
    num_failed = 0
    for case in case_list:
        if case == 3:
            src_path = rocal_data_path + "/rocal_data/multi_channel_wav"
            downmix = 1
        else:
            src_path = rocal_data_path + "/rocal_data/audio"
        print("\n\n")
        result = subprocess.run([build_folder_path + "/build/audio_tests", src_path, str(case), str(downmix), str(gpu), str(qa_mode)], stdout=subprocess.PIPE)    # nosec
        try:
            decoded_stdout = result.stdout.decode('utf-8')
        except UnicodeDecodeError as e:
            # Handle the error by replacing or ignoring problematic characters
            decoded_stdout = result.stdout.decode('utf-8', errors='replace')

        # check the QA status of the test case
        if "PASSED" in decoded_stdout:
            num_passed += 1
            passed_cases.append(test_case_augmentation_map[case])
        else:
            num_failed += 1
            failed_cases.append(test_case_augmentation_map[case])
        print(decoded_stdout)

    if qa_mode:
        print("Number of PASSED:", num_passed)
        print(passed_cases)
        print("Number of FAILED:", num_failed)
        print(failed_cases)
    print("------------------------------------------------------------------------------------------")

def audio_test_suite_parser_and_validator():
    parser = argparse.ArgumentParser()
    parser.add_argument("--gpu", type = int, default = 0, help = "Backend to run (1 - GPU / 0 - CPU)")
    parser.add_argument("--downmix", type = bool, default = False, help = "Runs the Audio with downMix option (True / False)")
    parser.add_argument("--test_case", type = int, default = None, help = "Testing case")
    parser.add_argument("--qa_mode", type = int, default = 1, help = "Compare outputs with reference outputs (0 - disabled / 1 - enabled)")
    args = parser.parse_args()
    return args

def main():
    script_path = os.path.dirname(os.path.realpath(__file__))
    rocal_data_path = os.environ.get("ROCAL_DATA_PATH")
    if rocal_data_path is None:
        print("Need to export ROCAL_DATA_PATH")
        sys.exit()

    sys.dont_write_bytecode = True
    build_folder_path = os.getcwd()

    args = audio_test_suite_parser_and_validator()
    gpu = args.gpu
    downmix = args.downmix
    test_case = args.test_case
    qa_mode = args.qa_mode

    case_list = list(test_case_augmentation_map.keys())

    if test_case is not None: 
        if test_case not in case_list:
            print(" Invalid Test Case! ")
            exit()
        else:
            case_list = [test_case]

    # Enable extglob
    if os.path.exists(build_folder_path + "/build"):
        shutil.rmtree(build_folder_path + "/build")
    os.makedirs(build_folder_path + "/build")
    os.chdir(build_folder_path + "/build")

    # Run cmake and make commands
    subprocess.run(["cmake", script_path], cwd=".")   # nosec
    subprocess.run(["make", "-j16"], cwd=".")    # nosec

    run_unit_test(rocal_data_path, qa_mode, gpu, downmix, build_folder_path, case_list)

if __name__ == "__main__":
    main()




import sys
from amd.rocal.pipeline import pipeline_def
from amd.rocal.plugin.generic import ROCALClassificationIterator
import amd.rocal.fn as fn
import amd.rocal.types as types
import matplotlib.gridspec as gridspec
import matplotlib.pyplot as plt
import numpy as np

seed = 1549361629
image_dir = "/opt/rocm/share/rocal/test/data/images/AMD-tinyDataSet"
batch_size = 4
gpu_id = 0

def show_images(image_batch):
    columns = 4
    rows = (batch_size + 1) // (columns)
    #fig = plt.figure(figsize = (32,(32 // columns) * rows))
    gs = gridspec.GridSpec(rows, columns)
    for j in range(rows*columns):
        #print('\n Display image: ', j)
        plt.subplot(gs[j])
        img = image_batch[j]
        plt.axis("off")
        plt.imshow(img)
    plt.show()


def show_pipeline_output(pipe, device):
    pipe.build()
    data_loader = ROCALClassificationIterator(pipe, device=device)
    images = next(iter(data_loader))
    show_images(images[0][0])

@pipeline_def(seed=seed)
def image_decoder_pipeline(device="cpu", path=image_dir):
    jpegs, labels = fn.readers.file(file_root=path)
    images = fn.decoders.image(jpegs, file_root=path, device=device, output_type=types.RGB, shard_id=0, num_shards=1, random_shuffle=False)
    return fn.resize(images, device=device, resize_width=300, resize_height=300)

def main():
    print ('Optional arguments: <cpu/gpu image_folder>')
    bs = batch_size
    rocal_device = "cpu"
    rocal_cpu = True
    img_folder = image_dir
    if  len(sys.argv) > 1:
      if(sys.argv[1] == "gpu"):
          rocal_device = "gpu"
          rocal_cpu = False
    if  len(sys.argv) > 2:
      img_folder = sys.argv[2]
    pipe = image_decoder_pipeline(batch_size=bs, num_threads=1, device_id=gpu_id, rocal_cpu=rocal_cpu, tensor_layout=types.NHWC, 
                                reverse_channels=True, mean = [0, 0, 0], std=[255,255,255], device=rocal_device, path=img_folder)
    show_pipeline_output(pipe, device=rocal_device)

if __name__ == '__main__':
    main()



# Copyright (c) 2018 - 2025 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from math import sqrt
import torch
import itertools
import os
import ctypes
from amd.rocal.pipeline import Pipeline
import amd.rocal.fn as fn
import amd.rocal.types as types
import numpy as np
from parse_config import parse_args


class ROCALCOCOIterator(object):
    """
    COCO ROCAL iterator for pyTorch.

    Parameters
    ----------
    pipelines : list of amd.rocal.pipeline.Pipeline
                List of pipelines to use
    size : int
           Epoch size.
    """

    def __init__(self, pipelines, tensor_layout=types.NCHW, reverse_channels=False, multiplier=None, offset=None, tensor_dtype=types.FLOAT, device="cpu", display=False):

        try:
            assert pipelines is not None, "Number of provided pipelines has to be at least 1"
        except Exception as ex:
            print(ex)
        self.loader = pipelines
        self.tensor_format = tensor_layout
        self.multiplier = multiplier if multiplier else [1.0, 1.0, 1.0]
        self.offset = offset if offset else [0.0, 0.0, 0.0]
        self.reverse_channels = reverse_channels
        self.tensor_dtype = tensor_dtype
        self.device = device
        self.device_id = self.loader._device_id
        self.bs = self.loader._batch_size
        self.output_list = self.dimensions = self.torch_dtype = None
        self.display = display
        # Image id of a batch of images
        self.image_id = np.zeros(self.bs, dtype="int32")
        # Count of labels/ bboxes in a batch
        self.bboxes_label_count = np.zeros(self.bs, dtype="int32")
        # Image sizes of a batch
        self.img_size = np.zeros((self.bs * 2), dtype="int32")
        self.output_memory_type = self.loader._output_memory_type

    def next(self):
        return self.__next__()

    def __next__(self):
        if self.loader.rocal_run() != 0:
            raise StopIteration
        else:
            self.output_tensor_list = self.loader.get_output_tensors()

        if self.output_list is None:
            self.output_list = []
            for i in range(len(self.output_tensor_list)):
                self.dimensions = self.output_tensor_list[i].dimensions()
                self.torch_dtype = self.output_tensor_list[i].dtype()
                if self.device == "cpu":
                    self.output = torch.empty(
                        self.dimensions, dtype=getattr(torch, self.torch_dtype))
                else:
                    torch_gpu_device = torch.device('cuda', self.device_id)
                    self.output = torch.empty(self.dimensions, dtype=getattr(
                        torch, self.torch_dtype), device=torch_gpu_device)
                self.output_tensor_list[i].copy_data(ctypes.c_void_p(
                    self.output.data_ptr()), self.output_memory_type)
                self.output_list.append(self.output)
        else:
            for i in range(len(self.output_tensor_list)):
                self.output_tensor_list[i].copy_data(ctypes.c_void_p(
                    self.output_list[i].data_ptr()), self.output_memory_type)

        self.labels = self.loader.get_bounding_box_labels()
        # 1D bboxes array in a batch
        self.bboxes = self.loader.get_bounding_box_cords()
        self.loader.get_image_id(self.image_id)
        image_id_tensor = torch.tensor(self.image_id)
        image_size_tensor = torch.tensor(self.img_size).view(-1, self.bs, 2)

        for i in range(self.bs):
            if self.display:
                img = self.output
                draw_patches(img[i], self.image_id[i],
                             self.bboxes[i], self.device, self.tensor_dtype, self.tensor_format)
        return (self.output), self.bboxes, self.labels, image_id_tensor, image_size_tensor

    def reset(self):
        self.loader.rocal_reset_loaders()

    def __iter__(self):
        return self


def draw_patches(img, idx, bboxes, device, dtype, layout):
    # image is expected as a tensor, bboxes as numpy
    import cv2
    if device == "cpu":
        image = img.detach().numpy()
    else:
        image = img.cpu().numpy()
    if dtype == types.FLOAT16:
        image = (image).astype('uint8')
    if layout == types.NCHW:
        image = image.transpose([1, 2, 0])
    image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)
    bboxes = np.reshape(bboxes, (-1, 4))

    for (l, t, r, b) in bboxes:
        loc_ = [l, t, r, b]
        color = (255, 0, 0)
        thickness = 2
        image = cv2.UMat(image).get()
        image = cv2.rectangle(image, (int(loc_[0]), int(loc_[1])), (int(
            (loc_[2])), int((loc_[3]))), color, thickness)
        cv2.imwrite("output_folder/coco_reader/" +
                    str(idx)+"_"+"train"+".png", image)


def main():
    args = parse_args()
    # Args
    image_path = args.image_dataset_path
    annotation_path = args.json_path
    rocal_cpu = False if args.rocal_gpu else True
    batch_size = args.batch_size
    display = args.display
    num_threads = args.num_threads
    local_rank = args.local_rank
    world_size = args.world_size
    random_seed = args.seed
    tensor_format = types.NHWC if args.NHWC else types.NCHW
    tensor_dtype = types.FLOAT16 if args.fp16 else types.FLOAT
    try:
        path = "output_folder/coco_reader/"
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
    except OSError as error:
        print(error)

    # Create Pipeline instance
    pipe = Pipeline(batch_size=batch_size, num_threads=num_threads, device_id=args.local_rank,
                    seed=random_seed, rocal_cpu=rocal_cpu, tensor_layout=tensor_format, tensor_dtype=tensor_dtype)
    # Use pipeline instance to make calls to reader, decoder & augmentation's
    with pipe:
        jpegs, bboxes, labels = fn.readers.coco(
            annotations_file=annotation_path)
        images_decoded = fn.decoders.image(jpegs, output_type=types.RGB, file_root=image_path,
                                           annotations_file=annotation_path, random_shuffle=False, shard_id=local_rank, num_shards=world_size)
        res_images = fn.resize(
            images_decoded, resize_width=300, resize_height=300)
        saturation = fn.uniform(range=[0.1, 0.4])
        contrast = fn.uniform(range=[0.1, 25.0])
        brightness = fn.uniform(range=[0.875, 1.125])
        hue = fn.uniform(range=[5.0, 170.0])
        ct_images = fn.color_twist(
            res_images, saturation=saturation, contrast=contrast, brightness=brightness, hue=hue)
        flip_coin = fn.random.coin_flip(probability=0.5)
        cmn_images = fn.crop_mirror_normalize(ct_images,
                                              crop=(224, 224),
                                              crop_pos_x=0.0,
                                              crop_pos_y=0.0,
                                              mean=[0, 0, 0],
                                              std=[1, 1, 1],
                                              mirror=flip_coin,
                                              output_layout=tensor_format,
                                              output_dtype=tensor_dtype)
        pipe.set_outputs(cmn_images)
    # Build the pipeline
    pipe.build()
    # Dataloader
    if (args.rocal_gpu):
        data_loader = ROCALCOCOIterator(
            pipe, multiplier=pipe._multiplier, offset=pipe._offset, display=display, tensor_layout=tensor_format, tensor_dtype=tensor_dtype, device="cuda")
    else:
        data_loader = ROCALCOCOIterator(
            pipe, multiplier=pipe._multiplier, offset=pipe._offset, display=display, tensor_layout=tensor_format, tensor_dtype=tensor_dtype, device="cpu")

    import timeit
    start = timeit.default_timer()
    # Enumerate over the Dataloader
    for epoch in range(int(args.num_epochs)):
        print("EPOCH:::::", epoch)
        for i, it in enumerate(data_loader, 0):
            if args.print_tensor:
                print("**************", i, "*******************")
                print("**************starts*******************")
                print("\nIMAGES : \n", it[0])
                print("\nBBOXES:\n", it[1])
                print("\nLABELS:\n", it[2])
                print("\nIMAGE ID:\n", it[3])
                print("\nIMAGE SIZE:\n", it[4])
                print("**************ends*******************")
                print("**************", i, "*******************")
        data_loader.reset()
    # Your statements here
    stop = timeit.default_timer()

    print('\n Time: ', stop - start)

    print("##############################  COCO READER  SUCCESS  ############################")


if __name__ == '__main__':
    main()



# Copyright (c) 2025 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from amd.rocal.pipeline import Pipeline
from amd.rocal.plugin.generic import ROCALGenericIterator
import amd.rocal.fn as fn
import sys
import os
import random

def draw_patches(img, idx):
    # image is expected as a numpy array
    import cv2
    img = img.transpose([1, 2, 0])
    image = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)
    cv2.imwrite("output_folder/cifar10_reader/" + str(idx) +
                "_" + "train" + ".png", image)

def main():
    if len(sys.argv) < 4:
        print('Please pass cifar10_folder cpu/gpu batch_size')
        exit(0)
    try:
        path = "output_folder/cifar10_reader/"
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
    except OSError as error:
        print(error)
    data_path = sys.argv[1]
    if (sys.argv[2] == "cpu"):
        rocal_cpu = True
    else:
        rocal_cpu = False
    batch_size = int(sys.argv[3])
    num_threads = 1
    device_id = 0
    local_rank = 0
    world_size = 1
    random_seed = random.SystemRandom().randint(0, 2**32 - 1)

    pipeline = Pipeline(batch_size=batch_size, num_threads=num_threads, device_id=device_id, seed=random_seed, rocal_cpu=rocal_cpu)

    with pipeline:
        cifar10_reader_output = fn.readers.cifar10(file_root=data_path, shard_id=local_rank, num_shards=world_size)
        pipeline.set_outputs(cifar10_reader_output)

    pipeline.build()

    cifar10IteratorPipeline = ROCALGenericIterator(pipeline)
    cnt = 0
    for epoch in range(1):
        print("+++++++++++++++++++++++++++++EPOCH+++++++++++++++++++++++++++++++++++++",epoch)
        for i , it in enumerate(cifar10IteratorPipeline):
            for img in it[0]:
                cnt += 1
                draw_patches(img[0], cnt)
        cifar10IteratorPipeline.reset()
    print("*********************************************************************")


if __name__ == '__main__':
    main()